// SYSTEM DEPENDENCES
# include <stdio.h>
# include <stdlib.h>
# include <stdbool.h>
# include <math.h>

// AUTOMAN DEPENDENCES
# include "automan/matrix.c"
# include "automan/map_jacobian_n.c"
# include "automan/gauss_solver.c"
# include "automan/periodic_orbit.c"

// USER CONTROLS
# include "example_specs.c"

// AUTOMAN
# include "automan/automan.c"

// USER FUNCTIONS
# include "example_map.c"

int main ()
{
    int period;
    manifold M;
    double a = 1.2;   // control param
    double x0[2];     // fixed point or member of upo

    // output manifolds file
    char out_man[] = "results/manifold_ur.dat";
    // output fixed point file
    FILE *f0 = fopen ("results/fixed_point.dat", "w");

    // initial guess for the periodic orbit coordinates
    x0[0] = 0.1;
    x0[1] = 0.1;

    // search orbit from the guess
    periodic_orbit (PERIOD, x0, &a);
    fprintf (f0, "%.16e  %16e\n", x0[0], x0[1]);

    // calculate manifold from x0
    trace_manifold (x0, &a, &M);

    // save manifold modes to file
    save_nodes (out_man, M);

    fclose (f0);
    return 0;
}
